@@ -3,33 +3,43 @@
 import asyncio
 import json
 from pathlib import Path
+from textwrap import dedent
 from typing import Annotated
 
 from fastmcp import Context, FastMCP
 from fastmcp.tools.tool import ToolResult
 from mcp.types import TextContent
 
-from ...providers.moesif_provider import add_moesif_middleware
+from ...providers.middleware_provider import get_mcp_middleware
+from ...providers.toolset_provider import get_mcp_tools
+from ...shared.utils.prompt_utils import get_feedback_prompt
 from .services.circuit_service import CircuitService
 from .services.notebook_service import NotebookService
+from .services.optimization_service import OptimizationService
 from .services.pdk_service import PdkService
 from .services.simulation_service import SimulationService
+from .services.statements_service import StatementsService
 from .utils.pdk import extract_pdk_type_from_code, flatten_pdks_response
-from .utils.wavelengths import get_default_wavelength_range, get_wavelengths_from_bounds, get_wavelengths_from_statements
+from .utils.wavelengths import resolve_wavelengths
 
 mcp = FastMCP(
     name="AxPhotonicsPreview Server",
-    instructions="""This server provides tools to design
-    and simulate photonic integrated circuits.""",
+    instructions=dedent(
+        """This server provides tools to design
+    and simulate photonic integrated circuits.
+    """
+        + get_feedback_prompt("design_circuit, simulate_circuit, list_available_pdks, get_pdk_info")
+    ),
     version="0.0.1",
+    middleware=get_mcp_middleware(),
+    tools=get_mcp_tools(),
 )
-
-add_moesif_middleware(mcp)
-
 circuit_service = CircuitService()
 simulation_service = SimulationService()
 notebook_service = NotebookService()
 pdk_service = PdkService()
+statements_service = StatementsService()
+optimization_service = OptimizationService()
 
 
 @mcp.tool(
@@ -90,7 +100,12 @@ async def design(
         json.dump(formalize_response, f)
 
     return ToolResult(
-        content=[TextContent(type="text", text=(f"Generated circuit at {circuit_file_path}, statements at {statements_file_path}"))],
+        content=[
+            TextContent(
+                type="text",
+                text=(f"Generated circuit at {circuit_file_path}, statements at {statements_file_path}.\n\n"),
+            )
+        ],
         structured_content={
             "circuit_file_path": str(circuit_file_path),
             "code": code,
@@ -106,6 +121,20 @@ async def design(
 )
 async def simulate_circuit(
     file_path: Annotated[Path, "The absolute path to the python file to analyze"],
+    statements_file_path: Annotated[
+        Path | None, "Optional path to a JSON statements file to define the wavelength range. Ignored if `wavelength_range` is provided."
+    ] = None,
+    wavelength_range: Annotated[
+        str | None,
+        (
+            "Wavelength range in the format 'start,end' (e.g. '1.3,1.6')."
+            " Providing this value overrides wavelength settings from `statements_file_path`."
+        ),
+    ] = None,
+    wavelength_points: Annotated[
+        str | None,
+        "Number of points for the custom wavelength range (e.g. 10000). Only used if `wavelength_range` is also specified.",
+    ] = None,
 ) -> ToolResult:
     """Simulates a circuit and saves the results in a Jupyter notebook.
 
@@ -114,6 +143,8 @@ async def simulate_circuit(
 
     Args:
         file_path: Path to the Python circuit file.
+        statements_file_path: Optional path to a JSON file to extract the wavelength range.
+        wavelength_range: Optional string for the simulation wavelengths.
 
     Returns:
         A ToolResult object with the path to the generated notebook and simulation output.
@@ -127,22 +158,13 @@ async def simulate_circuit(
     code = await asyncio.to_thread(file_path.read_bytes)
     netlist = await circuit_service.get_netlist_from_code(code)
     pdk_type = extract_pdk_type_from_code(code.decode("utf-8")) or "cspdk.si220.cband"
-    wavelengths = None
-    # TODO: figure out why accepting those values as inpout does not work on some systems
-    statements_file_path, wavelength_range = None, None
-    if statements_file_path:
-        if not statements_file_path.exists():
-            raise FileNotFoundError(f"Statement file not found: {file_path}")
-        statements_raw = await asyncio.to_thread(statements_file_path.read_bytes)
-        statement_dict = json.loads(statements_raw)
-        statements = statement_dict.get("statements", [])
-        wavelengths = get_wavelengths_from_statements(statements)
 
-    if wavelengths is None and wavelength_range:
-        wavelengths = get_wavelengths_from_bounds(*wavelength_range)
-
-    if wavelengths is None:
-        wavelengths = get_default_wavelength_range(pdk_type=pdk_type)
+    wavelengths = await resolve_wavelengths(
+        wavelength_range=wavelength_range,
+        wavelength_points=wavelength_points,
+        statements_file_path=statements_file_path,
+        pdk_type=pdk_type,
+    )
 
     response = await simulation_service.simulate_from_code(
         {
@@ -186,6 +208,109 @@ async def simulate_circuit(
     )
 
 
+@mcp.tool(
+    name="validate_statements",
+    description="Validates the statements in a photonic circuit design.",
+    tags=["validate", "statements"],
+)
+async def validate_statements(
+    file_path: Annotated[Path, "Path to the Python file containing the circuit code"],
+    statements_path: Annotated[Path, "Path to the JSON file containing the circuit statements"],
+) -> ToolResult:
+    """Validates the statements in a photonic circuit design."""
+    if not file_path.exists():
+        raise FileNotFoundError(f"Circuit code file not found: {file_path}")
+    if not statements_path.exists():
+        raise FileNotFoundError(f"Statements file not found: {statements_path}")
+    if not statements_path.read_text().strip():  # raise error if statements file is empty
+        raise ValueError(f"Statements file is empty: {statements_path}")
+
+    code = await asyncio.to_thread(file_path.read_bytes)
+    netlist = await circuit_service.get_netlist_from_code(code)
+
+    with statements_path.open("r", encoding="utf-8") as f:
+        statements_json = json.load(f)
+
+    statements_list = statements_json.get("statements", [])
+
+    request_body = {
+        "netlist": netlist,
+        "statements": statements_list,
+    }
+
+    verified_statements = await statements_service.validate_statements(request_body)
+
+    # Convert to pretty JSON
+    verified_statements_json = json.dumps(verified_statements, indent=2)
+
+    verified_statements_path = statements_path.parent / "verified_statements.json"
+    with verified_statements_path.open("w", encoding="utf-8") as f:
+        f.write(verified_statements_json)
+
+    return ToolResult(
+        content=[
+            TextContent(
+                type="text",
+                text=f"Verified statements saved at {verified_statements_path}",
+            )
+        ],
+        structured_content={
+            "verified_statements_path": str(verified_statements_path),
+            "verified_statements": verified_statements,
+        },
+    )
+
+
+@mcp.tool(
+    name="optimize_circuit",
+    description="Optimizes a photonic circuit by refining the generated code using its statements.",
+    tags=["optimize", "gfsfactory"],
+)
+async def optimize_circuit(
+    code_path: Annotated[Path, "Path to the Python file containing the circuit code"],
+    statements_path: Annotated[Path, "Path to the JSON file containing the circuit statements"],
+) -> ToolResult:
+    """Optimize a photonic integrated circuit."""
+    if not code_path.exists():
+        raise FileNotFoundError(f"Circuit code not found: {code_path}")
+    if not statements_path.exists():
+        raise FileNotFoundError(f"Statements file not found: {statements_path}")
+
+    code = code_path.read_text(encoding="utf-8")
+
+    with statements_path.open("r", encoding="utf-8") as f:
+        statements_json = json.load(f)
+
+    statements_list = statements_json.get("statements", [])
+
+    request_body = {
+        "code": code,
+        "statements": statements_list,
+    }
+
+    response = await optimization_service.optimize_code(request_body)
+
+    optimized_code = response.get("optimized_code", "")
+
+    optimized_file_path = code_path.parent / f"{code_path.stem}_optimized.py"
+
+    with optimized_file_path.open("w", encoding="utf-8") as f:
+        f.write(optimized_code)
+
+    return ToolResult(
+        content=[
+            TextContent(
+                type="text",
+                text=f"Optimized circuit saved at {optimized_file_path}",
+            )
+        ],
+        structured_content={
+            "optimized_file_path": str(optimized_file_path),
+            "optimized_code": optimized_code,
+        },
+    )
+
+
 @mcp.tool(
     name="list_available_pdks",
     description="Get a list of all available PDKs that the user has access to.",
